// ======= ANIMATE CONTROL
// ======= for DELPHI 32, Win95
// ======= by Igor Lookin, Russia, Barnaul
// ======= (3852)232-948 (voice in Russia)
// ======= E-mail: sas@mezon.altai.su
// ======= FidoNet 2:5002/27.2

// Animation control is a window that silently displays
// an Audio Video Interleaved (AVI) clip.
// An AVI clip is a series of bitmap frames like a movie.
// Although AVI clips can have sound, you cannot use such
// clips with animation controls.
// You can use only silent AVI clips. (Win32API)

unit Animate;

interface

uses CommCtrl,Windows,Classes,Controls,Forms,Messages,SysUtils;

type
  TBorderOptions = (brBorder, brStaticEdge, brClientEdge);

  TBorderInfo = set of TBorderOptions;

  TAnimation = class(TWinControl)
  private
    FParentHandle : HWnd;
    FAniObjectInstance : pointer;
    FParentMainWndProc : pointer;
    FBorderOptions: TBorderInfo;
    FOpen: Boolean;
    FPlay: Boolean;
    FCenter: Boolean;
    FTo: Integer;
    FFrom: Integer;
    FRepeat: Integer;
    FTransParent : Boolean;
    FAniName : string;
    FOnStartPlay: TNotifyEvent;
    FOnStopPlay: TNotifyEvent;
  protected
    procedure CreateWnd; override;
    procedure NewMainWndProc(var Message: TMessage);
    procedure SetStyle(var Style,ExStyle : Longint);
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure SetBorderStyle(Value: TBorderInfo);
    procedure SetCenter( Value: boolean );
    procedure SetTransParent( Value: boolean );
    procedure SetRepeat( Value: Integer);
    procedure SetFrom( Value: Integer);
    procedure SetTo( Value: Integer);
    procedure SetAniName(Value: string);
    procedure SetPlay(Value: boolean);
    procedure RePlay;
    procedure Animate_Open;
    procedure Animate_Play;
    procedure Animate_Stop;
    procedure Animate_Close;
    function  Animate_Seek(frame: integer) : boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BorderStyle: TBorderInfo read FBorderOptions write SetBorderStyle default [brStaticEdge,brClientEdge];
    property Enabled;
    property Color;
    property ParentColor;
    property TabOrder;
    property TabStop;
    property Visible;
    property Align;
//
    property OnEnter;
    property OnExit;
//
    property OnStartPlay: TNotifyEvent read FOnStartPlay write FOnStartPlay;
    property OnStopPlay: TNotifyEvent read FOnStopPlay write FOnStopPlay;
//
    procedure AniSeek(wFrame : integer);
    property  AniPlay : boolean read FPlay write SetPlay;
    property  AniName : string read FAniName write SetAniName;
    property  AniStart: Integer read FFrom write SetFrom default 0;
    property  AniEnd: Integer read FTo write SetTo default -1;
    property  AniRepeat : Integer read FRepeat write SetRepeat default -1;
    property  AniTransParent : Boolean read FTransParent write SetTransParent default true;
    property  AniCenter: Boolean read FCenter write SetCenter default true;
  end;

procedure Register;

implementation

// Bug in Animate_Open function :
//
// Project xxxx.exe raised exception class EAccessViolation with
// message 'Access violation at address ########. Read of address
// ########'.
// procedure TWinControl.MainWndProc(var Message: TMessage);
// begin
//   try
//     try
// >     WndProc(Message);
//     finally
//       FreeDeviceContexts;
//       FreeMemoryContexts;
//     end;
//   except
//     Application.HandleException(Self);
//   end;
// end;
procedure TAnimation.Animate_Open;
begin
  if WindowHandle <> 0 then
     FOpen := SendMessage(WindowHandle,ACM_OPEN, 0, Longint(FAniName)) <> 0;
end;

procedure TAnimation.Animate_Play;
begin
  if WindowHandle <> 0 then
     FPlay := SendMessage(WindowHandle,ACM_PLAY, FRepeat, MakeLong(FFrom, FTo)) <> 0;
end;

procedure TAnimation.Animate_Stop;
begin
  if WindowHandle <> 0 then
     FPlay := SendMessage(WindowHandle,ACM_STOP, 0, 0) = 0;
end;

procedure TAnimation.Animate_Close;
begin
  if WindowHandle <> 0 then
     FOpen := SendMessage(WindowHandle,ACM_OPEN, 0, 0) = 0;
end;

function TAnimation.Animate_Seek(frame: integer) : boolean;
begin
  if WindowHandle <> 0 then
     Result := SendMessage(WindowHandle,ACM_PLAY, 1, MakeLong(frame, frame)) <> 0;
end;

procedure TAnimation.CreateWnd;
begin
  Animate_Close;
  inherited CreateWnd;
// parent message procedure
  if (FParentHandle = 0) and (WindowHandle <> 0) then
  begin
    FParentHandle := GetWindowLong(WindowHandle, GWL_HWNDPARENT);
    if FParentHandle <> 0 then
    begin
      FAniObjectInstance := MakeObjectInstance(NewMainWndProc);
      FParentMainWndProc :=
         Pointer(GetWindowLong(FParentHandle,GWL_WNDPROC));
      SetWindowLong(FParentHandle, GWL_WNDPROC,
                    Longint(FAniObjectInstance));
    end;
  end;
//
  Animate_open;
  RePlay;
end;

procedure TAnimation.SetStyle(var Style,ExStyle : Longint);
const
  BorderStyles: array [TBorderOptions] of integer = (
       WS_BORDER, WS_EX_STATICEDGE, WS_EX_CLIENTEDGE);
var
  Option: TBorderOptions;
begin
  for Option := brStaticEdge to High(Option) do
     if Option in FBorderOptions then
          ExStyle := ExStyle or BorderStyles[Option]
     else ExStyle := ExStyle and (not BorderStyles[Option]);
  if FCenter then Style := Style or ACS_CENTER
  else Style := Style and (not ACS_CENTER);
  if FTransParent then Style := Style or ACS_TRANSPARENT
  else Style := Style and (not ACS_TRANSPARENT);
  Style := Style or WS_CHILD;
  if  brBorder in FBorderOptions then Style := Style or WS_BORDER;
end;

procedure TAnimation.CreateParams(var Params: TCreateParams);
begin
  InitCommonControls;
  inherited CreateParams(Params);
  CreateSubClass(Params, ANIMATE_CLASS);
  SetStyle(Params.Style,Params.ExStyle);
end;

constructor TAnimation.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [];
  Width  := 64;
  Height := 64;
  FFrom  := 0;
  FTo    := -1;
  FRepeat:= -1;
  FCenter:= true;
  FTransParent:= true;
  FAniName := '';
  FPlay := false;
  FOpen := false;
  FBorderOptions := [brStaticEdge,brClientEdge];
  Color := clBtnFace;
  TabStop := false;
// for exchenge parent message
  FParentMainWndProc := nil;
  FAniObjectInstance := nil;
  FParentHandle := 0;
end;

destructor TAnimation.Destroy;
begin
  if FAniObjectInstance <> nil then
     FreeObjectInstance(FAniObjectInstance);
  if FParentHandle <> 0 then
    SetWindowLong(FParentHandle, GWL_WNDPROC,
                  Longint(FParentMainWndProc));
  inherited Destroy;
end;

procedure TAnimation.NewMainWndProc(var Message: TMessage);
begin
  with TMessage(Message) do
  begin
    if Msg = WM_COMMAND then
       case HIWORD(WParam) of
          ACN_START :
             begin
               if Assigned(FOnStartPlay) then FOnStartPlay(Self);
               Exit;
             end;
          ACN_STOP  :
             begin
               if Assigned(FOnStopPlay) then FOnStopPlay(Self);
               Exit;
             end;
       end;
    if FParentMainWndProc <> nil then
       Result := CallWindowProc(FParentMainWndProc,FParentHandle, Msg, WParam, LParam);
  end;
end;

procedure TAnimation.CMColorChanged(var Message: TMessage);
begin
  inherited;
  RecreateWnd;
  RePlay;
end;

procedure TAnimation.SetBorderStyle(Value: TBorderInfo);
begin
  if BorderStyle <> Value then
  begin
    FBorderOptions := Value;
    RecreateWnd;
    RePlay;
  end;
end;

procedure TAnimation.SetCenter( Value: boolean );
begin
  if FCenter <> Value then
  begin
    FCenter := Value;
    RecreateWnd;
    RePlay;
  end;
end;

procedure TAnimation.SetTransParent( Value: boolean );
begin
  if FTransParent <> Value then
  begin
    FTransParent := Value;
    RecreateWnd;
    RePlay;
 end;
end;

procedure TAnimation.SetPlay(Value: boolean);
begin
  if not FOpen then Animate_open;
  if Value then Animate_Play else Animate_Stop;
end;

procedure TAnimation.SetAniName(Value: string);
begin
  FAniName := Value;
  FPlay := false;
  Animate_Close;
  Animate_open;
end;

procedure TAnimation.RePlay;
begin
  if FPlay then SetPlay(true);
end;

procedure TAnimation.SetRepeat( Value: Integer);
begin
  if FRepeat <> Value then
  begin
    FRepeat := Value;
    RePlay;
  end;
end;

procedure TAnimation.SetTo( Value: Integer);
begin
  if FTo <> Value then
  begin
     FTo := Value;
     RePlay;
  end;
end;

procedure TAnimation.SetFrom( Value: Integer);
begin
  if FFrom <> Value then
  begin
     FFrom := Value;
     Animate_Seek(FFrom);
  end;
end;

procedure TAnimation.AniSeek(wFrame : integer);
begin
  Animate_Seek(wframe);
end;

procedure Register;
begin
  RegisterComponents('Win95', [TAnimation]);
end;

end.
